@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2014 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -97,8 +97,7 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
@@ -119,7 +118,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
  * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
@@ -130,7 +129,6 @@
 
 #include "nsock.h"
 #include <vector>
-#include "nmap.h"
 #include "libnetutil/npacket.h"
 
 /* Mention some classes here so we don't have to place the declarations in
@@ -141,6 +139,7 @@ class FPProbe;
 
 class Target;
 class FingerPrintResultsIPv6;
+struct FingerMatch;
 
 /******************************************************************************
  * CONSTANT DEFINITIONS                                                       *
@@ -354,6 +353,7 @@ class FPProbe : public FPPacket {
   int setFailed();
   bool isTimed() const;
   int setTimed();
+  int changeSourceAddress(struct in6_addr *addr);
 
 };
 
@@ -393,6 +393,7 @@ class FPHost {
   unsigned int probes_sent;       /* Number of FPProbes sent (not counting retransmissions)       */
   unsigned int probes_answered;   /* Number of FPResponses received                               */
   unsigned int probes_unanswered; /* Number of FPProbes that timedout (after all retransmissions) */
+  bool incomplete_fp;             /* True if we were unable to send all attempted probes          */
   bool detection_done;            /* True if the OS detection process has been completed.         */
   bool timedprobes_sent;          /* True if the probes that have timing requirements were sent   */
   Target *target_host;            /* Info about the host to fingerprint                           */
@@ -425,6 +426,7 @@ class FPHost {
   virtual int schedule() = 0;
   virtual int callback(const u8 *pkt, size_t pkt_len, const struct timeval *tv) = 0;
   const struct sockaddr_storage *getTargetAddress();
+  void fail_one_probe();
 
 };
 
@@ -471,3 +473,4 @@ std::vector<FingerMatch> load_fp_matches();
 
 
 #endif /* __FPENGINE_H__ */
+
