@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2014 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -97,8 +97,7 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
@@ -119,7 +118,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
  * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 /* This code was originally part of the Nping tool.                        */
@@ -568,7 +567,7 @@ u16 IPv4Header::getSum() const {
 int IPv4Header::setDestinationAddress(u32 d){
   h.ip_dst.s_addr =  d;
   return OP_SUCCESS;
-} /* End of getDestinationAddress() */
+} /* End of setDestinationAddress() */
 
 /** Sets destination IP address.
  *  @warning Destination IP must be supplied in NETWORK byte order. Usually
@@ -577,7 +576,7 @@ int IPv4Header::setDestinationAddress(u32 d){
 int IPv4Header::setDestinationAddress(struct in_addr d){
   h.ip_dst=d;
   return OP_SUCCESS;
-} /* End of getDestinationAddress() */
+} /* End of setDestinationAddress() */
 
 
 /** Returns destination IP address.
@@ -682,7 +681,8 @@ const u8 *IPv4Header::getOpts() const {
 const u8 *IPv4Header::getOpts(int *len) const {
   if(len==NULL)
     printf("getOpts(): NULL pointer supplied.\n");
-  *len=ipoptlen;
+  else
+    *len=ipoptlen;
   return h.options;
 } /* End of getOpts() */
 
