@@ -2,7 +2,7 @@
  * ncat_listen.c -- --listen mode.                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -118,7 +118,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_listen.c 35435 2015-11-15 14:18:33Z dmiller $ */
+/* $Id: ncat_listen.c 36055 2016-07-30 00:13:18Z dmiller $ */
 
 #include "ncat.h"
 
@@ -532,6 +532,14 @@ static void post_handle_connection(struct fdinfo sinfo)
      * to our descriptor list or set.
      */
     if (o.cmdexec) {
+#ifdef HAVE_OPENSSL
+      /* We added this in handle_connection, but at this point the ssl
+       * connection has taken over. Stop tracking.
+       */
+      if (o.ssl) {
+        rm_fd(&client_fdlist, sinfo.fd);
+      }
+#endif
         if (o.keepopen)
             netrun(&sinfo, o.cmdexec);
         else
@@ -546,10 +554,13 @@ static void post_handle_connection(struct fdinfo sinfo)
             /* add it to our list of fds for maintaining maxfd */
 #ifdef HAVE_OPENSSL
             /* Don't add it twice (see handle_connection above) */
-            if (!o.ssl)
+            if (!o.ssl) {
 #endif
             if (add_fdinfo(&client_fdlist, &sinfo) < 0)
                 bye("add_fdinfo() failed.");
+#ifdef HAVE_OPENSSL
+            }
+#endif
         }
         FD_SET(sinfo.fd, &master_broadcastfds);
         if (add_fdinfo(&broadcast_fdlist, &sinfo) < 0)
