@@ -3,7 +3,7 @@
  * to mode-specific functions.                                             *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -119,7 +119,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_main.c 35420 2015-11-10 21:07:30Z dmiller $ */
+/* $Id: ncat_main.c 36302 2016-09-21 20:04:16Z dmiller $ */
 
 #include "nsock.h"
 #include "ncat.h"
@@ -334,7 +334,7 @@ int main(int argc, char *argv[])
     while (1) {
         /* handle command line arguments */
         int option_index;
-        int c = getopt_long(argc, argv, "46UCc:e:g:G:i:km:hp:d:lo:x:ts:uvw:n",
+        int c = getopt_long(argc, argv, "46UCc:e:g:G:i:km:hp:d:lo:x:ts:uvw:nz",
                             long_options, &option_index);
 
         /* That's the end of the options. */
@@ -458,6 +458,9 @@ int main(int argc, char *argv[])
         case 't':
             o.telnet = 1;
             break;
+        case 'z':
+            o.zerobyte = 1;
+            break;
         case 0:
             if (strcmp(long_options[option_index].name, "version") == 0) {
                 print_banner();
@@ -606,6 +609,7 @@ int main(int argc, char *argv[])
 "      --sctp                 Use SCTP instead of default TCP\n"
 "  -v, --verbose              Set verbosity level (can be used several times)\n"
 "  -w, --wait <time>          Connect timeout\n"
+"  -z                         Zero-I/O mode, report connection status only\n"
 "      --append-output        Append rather than clobber specified output files\n"
 "      --send-only            Only send data, ignoring received; quit on EOF\n"
 "      --recv-only            Only receive data, never send anything\n"
@@ -723,6 +727,16 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (o.zerobyte) {
+      if (o.listen)
+        bye("Services designed for LISTENING can't be used with -z");
+      if (o.telnet)
+        bye("Invalid option combination: -z and -t.");
+      if (o.execmode||o.cmdexec)
+        bye("Command execution can't be done along with option -z.");
+      if (!o.idletimeout && o.proto == IPPROTO_UDP)
+        o.idletimeout = 2 * 1000;
+    }
     /* Default port */
     if (o.listen && o.proxytype && !o.portno && srcport == -1)
         o.portno = DEFAULT_PROXY_PORT;
