@@ -253,7 +253,11 @@ static int set_dNSNames(X509 *cert, const struct lstr dNSNames[])
         if (gen_name == NULL)
             goto stack_err;
         gen_name->type = GEN_DNS;
+    #if OPENSSL_VERSION_NUMBER < 0x10100000L
         gen_name->d.dNSName = M_ASN1_IA5STRING_new();
+    #else
+        gen_name->d.dNSName = ASN1_IA5STRING_new();
+    #endif
         if (gen_name->d.dNSName == NULL)
             goto name_err;
         if (ASN1_STRING_set(gen_name->d.dNSName, name->s, name->len) == 0)
@@ -285,8 +289,9 @@ stack_err:
 static int gen_cert(X509 **cert, EVP_PKEY **key,
     const struct lstr commonNames[], const struct lstr dNSNames[])
 {
-    RSA *rsa;
-    int rc;
+    RSA *rsa = NULL;
+    BIGNUM *bne = NULL;
+    int rc, ret=0;
 
     *cert = NULL;
     *key = NULL;
@@ -296,9 +301,17 @@ static int gen_cert(X509 **cert, EVP_PKEY **key,
     if (*key == NULL)
         goto err;
     do {
-        rsa = RSA_generate_key(KEY_BITS, RSA_F4, NULL, NULL);
-        if (rsa == NULL)
+        /* Generate RSA key. */
+        bne = BN_new();
+        ret = BN_set_word(bne, RSA_F4);
+        if (ret != 1)
             goto err;
+
+        rsa = RSA_new();
+        ret = RSA_generate_key_ex(rsa, KEY_BITS, bne, NULL);
+        if (ret != 1)
+            goto err;
+        /* Check RSA key. */
         rc = RSA_check_key(rsa);
     } while (rc == 0);
     if (rc == -1)
