@@ -1,12 +1,12 @@
 
 /***************************************************************************
  * winfix.cc -- A few trivial windows-compatibility-related functions that *
- * are specific to Nmap.  Most of this has been moved into nbase so it can *
+ * are specific to Nping.  Most of this has been moved into nbase so it can *
  * be shared.                                                              *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -143,6 +143,10 @@
 #define DLI_ERROR VcppException(ERROR_SEVERITY_ERROR, ERROR_MOD_NOT_FOUND)
 #endif
 
+#define PCAP_DRIVER_NONE 0
+#define PCAP_DRIVER_WINPCAP 1
+#define PCAP_DRIVER_NPCAP 2
+
 extern NpingOps o;
 
 /*   internal functions   */
@@ -150,7 +154,7 @@ static void win_cleanup(void);
 static char pcaplist[4096];
 
 /* The code that has no preconditions to being called, so it can be
-   executed before even Nmap options parsing (so o.debugging and the
+   executed before even Nping options parsing (so o.getDebugging() and the
    like don't need to be used.  Its main function is to do
    WSAStartup() as some of the option parsing code does DNS
    resolution */
@@ -163,14 +167,15 @@ void win_pre_init() {
 		fatal("failed to start winsock.\n");
 }
 
-/* Check if the NPF service is running on Windows, and try to start it if it's
+/* Check if the NPCAP service is running on Windows, and try to start it if it's
    not. Return true if it was running or we were able to start it, false
    otherwise. */
-static bool start_npf() {
+static bool start_service(const char *svcname) {
   SC_HANDLE scm, npf;
   SERVICE_STATUS service;
   bool npf_running;
   int ret;
+  char startsvc[32];
 
   scm = NULL;
   npf = NULL;
@@ -180,13 +185,15 @@ static bool start_npf() {
     error("Error in OpenSCManager");
     goto quit_error;
   }
-  npf = OpenService(scm, "npf", SC_MANAGER_CONNECT | SERVICE_QUERY_STATUS);
+  npf = OpenService(scm, svcname, SC_MANAGER_CONNECT | SERVICE_QUERY_STATUS);
   if (npf == NULL) {
     error("Error in OpenService");
     goto quit_error;
   }
   if (!QueryServiceStatus(npf, &service)) {
+    /* No need to warn at this point: we'll check later
     error("Error in QueryServiceStatus");
+    */
     goto quit_error;
   }
   npf_running = (service.dwCurrentState & SERVICE_RUNNING) != 0;
@@ -195,19 +202,20 @@ static bool start_npf() {
 
   if (npf_running) {
     if (o.getDebugging() > DBG_1)
-      printf("NPF service is already running.\n");
+      printf("%s service is already running.\n", svcname);
     return true;
   }
 
-  /* NPF is not running. Try to start it. */
+  /* Service is not running. Try to start it. */
 
   if (o.getDebugging() > DBG_1)
-    printf("NPF service is not running.\n");
+    printf("%s service is already running.\n", svcname);
 
-  ret = (int) ShellExecute(0, "runas", "net.exe", "start npf", 0, SW_HIDE);
+  Snprintf(startsvc, 32, "start %s", svcname);
+  ret = (int) ShellExecute(0, "runas", "net.exe", startsvc, 0, SW_HIDE);
   if (ret <= 32) {
-    error("Unable to start NPF service: ShellExecute returned %d.\n\
-Resorting to unprivileged (non-administrator) mode.", ret);
+    error("Unable to start %s service: ShellExecute returned %d.\n\
+Resorting to unprivileged (non-administrator) mode.", svcname, ret);
     return false;
   }
 
@@ -215,9 +223,9 @@ Resorting to unprivileged (non-administrator) mode.", ret);
 
 quit_error:
   if (scm != NULL)
-    CloseHandle(scm);
+    CloseServiceHandle(scm);
   if (npf != NULL)
-    CloseHandle(npf);
+    CloseServiceHandle(npf);
 
   return false;
 }
@@ -246,6 +254,27 @@ static void init_dll_path()
 	}
 }
 
+/* If we find the Npcap driver, allow Nping to load Npcap DLLs from the "\System32\Npcap" directory. */
+static void init_npcap_dll_path()
+{
+	BOOL(WINAPI *SetDllDirectory)(LPCTSTR);
+	char sysdir_name[512];
+	int len;
+
+	SetDllDirectory = (BOOL(WINAPI *)(LPCTSTR)) GetProcAddress(GetModuleHandle("kernel32.dll"), "SetDllDirectoryA");
+	if (SetDllDirectory == NULL) {
+		pfatal("Error in SetDllDirectory");
+	}
+	else {
+		len = GetSystemDirectory(sysdir_name, 480);	//	be safe
+		if (!len)
+			pfatal("Error in GetSystemDirectory (%d)", GetLastError());
+		strcat(sysdir_name, "\\Npcap");
+		if (SetDllDirectory(sysdir_name) == 0)
+			pfatal("Error in SetDllDirectory(\"System32\\Npcap\")");
+	}
+}
+
 /* Requires that win_pre_init() has already been called, also that
    options processing has been done so that o.debugging is
    available */
@@ -258,6 +287,7 @@ void win_init()
 	PMIB_IPADDRTABLE pIp = 0;
 	int i;
 	int numipsleft;
+	int pcap_driver = PCAP_DRIVER_NONE;
 
 	init_dll_path();
 
@@ -282,7 +312,28 @@ void win_init()
     DWORD wait;
 		ULONG len = sizeof(pcaplist);
 
-		if(o.getDebugging() >= DBG_2) printf("Trying to initialize WinPcap\n");
+    if(o.getDebugging() >= DBG_2) printf("Trying to initialize Windows pcap engine\n");
+
+    /* o.getIsRoot() will be false at this point if the user asked for
+       --unprivileged. In that case don't bother them with a
+       potential UAC dialog when starting NPF. */
+    if (o.isRoot()) {
+      if (start_service("npcap"))
+        pcap_driver = PCAP_DRIVER_NPCAP;
+      else if (start_service("npf"))
+        pcap_driver = PCAP_DRIVER_WINPCAP;
+      else {
+        if(o.getDebugging() >= DBG_0) {
+          error("Unable to start either npcap or npf service");
+        }
+        pcap_driver = PCAP_DRIVER_NONE;
+        o.setHavePcap(false);
+      }
+    }
+
+    if (pcap_driver == PCAP_DRIVER_NPCAP)
+      init_npcap_dll_path();
+
     pcapMutex = CreateMutex(NULL, 0, "Global\\DnetPcapHangAvoidanceMutex");
     wait = WaitForSingleObject(pcapMutex, INFINITE);
 		PacketGetAdapterNames(pcaplist, &len);
@@ -294,22 +345,17 @@ void win_init()
 #ifdef _MSC_VER
 		if(FAILED(__HrLoadAllImportsForDll("wpcap.dll")))
 		{
-			error("WARNING: your winpcap is too old to use.  Nmap may not function.\n");
+			error("WARNING: your winpcap is too old to use.  Nping may not function.\n");
 			o.setHavePcap(false);
 		}
 #endif
 		if(o.getDebugging() >= DBG_1)
 			printf("Winpcap present, dynamic linked to: %s\n", pcap_lib_version());
 
-		/* o.getIsRoot() will be false at this point if the user asked for
-		   --unprivileged. In that case don't bother them with a
-		   potential UAC dialog when starting NPF. */
-		if (o.isRoot())
-			o.setHavePcap(o.havePcap() && start_npf());
 	}
 #ifdef _MSC_VER
 	__except (1) {
-			error("WARNING: Could not import all necessary WinPcap functions.  You may need to upgrade to version 3.1 or higher from http://www.winpcap.org.  Resorting to connect() mode -- Nmap may not function completely");
+			error("WARNING: Could not import all necessary Npcap functions.  You may need to upgrade to version 0.07 or higher from http://www.npcap.org.  Resorting to connect() mode -- Nping may not function completely");
 		o.setHavePcap(false);
 		}
 #endif
