@@ -4,14 +4,18 @@
 -- These functions will build strings and process buffers. Socket communication
 -- is left to the script to implement.
 --
--- @author "Daniel Miller <bonsaiviking@gmail.com>"
+-- @author Daniel Miller
 
 local stdnse = require "stdnse"
-local bin = require "bin"
+local string = require "string"
+local math = require "math"
 local os = require "os"
 local table = require "table"
 _ENV = stdnse.module("tls", stdnse.seeall)
 
+local pack = string.pack
+local unpack = string.unpack
+
 -- Most of the values in the tables below are from:
 -- http://www.iana.org/assignments/tls-parameters/
 PROTOCOLS = {
@@ -20,6 +24,7 @@ PROTOCOLS = {
   ["TLSv1.1"]     = 0x0302,
   ["TLSv1.2"]     = 0x0303
 }
+HIGHEST_PROTOCOL = "TLSv1.2"
 
 --
 -- TLS Record Types
@@ -68,6 +73,7 @@ TLS_ALERT_REGISTRY = {
   ["protocol_version"]                    = 70,
   ["insufficient_security"]               = 71,
   ["internal_error"]                      = 80,
+  ["inappropriate_fallback"]              = 86,
   ["user_canceled"]                       = 90,
   ["no_renegotiation"]                    = 100,
   ["unsupported_extension"]               = 110,
@@ -150,6 +156,24 @@ EC_POINT_FORMATS = {
   ansiX962_compressed_char2 = 2,
 }
 
+---
+-- RFC 5246 section 7.4.1.4.1. Signature Algorithms
+HashAlgorithms = {
+  none = 0,
+  md5 = 1,
+  sha1 = 2,
+  sha224 = 3,
+  sha256 = 4,
+  sha384 = 5,
+  sha512 = 6,
+}
+SignatureAlgorithms = {
+  anonymous = 0,
+  rsa = 1,
+  dsa = 2,
+  ecdsa = 3,
+}
+
 ---
 -- Extensions
 -- RFC 6066, draft-agl-tls-nextprotoneg-03
@@ -190,7 +214,7 @@ EXTENSION_HELPERS = {
   ["server_name"] = function (server_name)
     -- Only supports host_name type (0), as per RFC
     -- Support for other types could be added later
-    return bin.pack(">P", bin.pack(">CP", 0, server_name))
+    return pack(">s2", pack(">Bs2", 0, server_name))
   end,
   ["max_fragment_length"] = tostring,
   ["client_certificate_url"] = tostring,
@@ -200,16 +224,26 @@ EXTENSION_HELPERS = {
   ["elliptic_curves"] = function (elliptic_curves)
     local list = {}
     for _, name in ipairs(elliptic_curves) do
-      list[#list+1] = bin.pack(">S", ELLIPTIC_CURVES[name])
+      list[#list+1] = pack(">I2", ELLIPTIC_CURVES[name])
     end
-    return bin.pack(">P", table.concat(list))
+    return pack(">s2", table.concat(list))
   end,
   ["ec_point_formats"] = function (ec_point_formats)
     local list = {}
     for _, format in ipairs(ec_point_formats) do
-      list[#list+1] = bin.pack(">C", EC_POINT_FORMATS[format])
+      list[#list+1] = pack(">B", EC_POINT_FORMATS[format])
     end
-    return bin.pack(">p", table.concat(list))
+    return pack(">s1", table.concat(list))
+  end,
+  ["signature_algorithms"] = function(signature_algorithms)
+    local list = {}
+    for _, pair in ipairs(signature_algorithms) do
+      list[#list+1] = pack(">BB",
+        HashAlgorithms[pair[1]] or pair[1],
+        SignatureAlgorithms[pair[2]] or pair[2]
+        )
+    end
+    return pack(">s2", table.concat(list))
   end,
   ["next_protocol_negotiation"] = tostring,
 }
@@ -325,19 +359,19 @@ CIPHERS = {
 ["TLS_DHE_RSA_WITH_AES_256_CBC_SHA256"]            =  0x006B,
 ["TLS_DH_anon_WITH_AES_128_CBC_SHA256"]            =  0x006C,
 ["TLS_DH_anon_WITH_AES_256_CBC_SHA256"]            =  0x006D,
-["TLS_DHE_DSS_WITH_3DES_EDE_CBC_RMD"]       =  0x0072,  --draft-ietf-tls-openpgp-keys-05
-["TLS_DHE_DSS_WITH_AES_128_CBC_RMD"]        =  0x0073,  --draft-ietf-tls-openpgp-keys-05
-["TLS_DHE_DSS_WITH_AES_256_CBC_RMD"]        =  0x0074,  --draft-ietf-tls-openpgp-keys-05
-["TLS_DHE_RSA_WITH_3DES_EDE_CBC_RMD"]       =  0x0077,  --draft-ietf-tls-openpgp-keys-05
-["TLS_DHE_RSA_WITH_AES_128_CBC_RMD"]        =  0x0078,  --draft-ietf-tls-openpgp-keys-05
-["TLS_DHE_RSA_WITH_AES_256_CBC_RMD"]        =  0x0079,  --draft-ietf-tls-openpgp-keys-05
-["TLS_RSA_WITH_3DES_EDE_CBC_RMD"]           =  0x007C,  --draft-ietf-tls-openpgp-keys-05
-["TLS_RSA_WITH_AES_128_CBC_RMD"]            =  0x007D,  --draft-ietf-tls-openpgp-keys-05
-["TLS_RSA_WITH_AES_256_CBC_RMD"]            =  0x007E,  --draft-ietf-tls-openpgp-keys-05
-["TLS_GOSTR341094_WITH_28147_CNT_IMIT"]     =  0x0080,  --draft-chudov-cryptopro-cptls-04
-["TLS_GOSTR341001_WITH_28147_CNT_IMIT"]     =  0x0081,  --draft-chudov-cryptopro-cptls-04
-["TLS_GOSTR341094_WITH_NULL_GOSTR3411"]     =  0x0082,  --draft-chudov-cryptopro-cptls-04
-["TLS_GOSTR341001_WITH_NULL_GOSTR3411"]     =  0x0083,  --draft-chudov-cryptopro-cptls-04
+["TLS_DHE_DSS_WITH_3DES_EDE_CBC_RMD-draft"]       =  0x0072,  --draft-ietf-tls-openpgp-keys-05
+["TLS_DHE_DSS_WITH_AES_128_CBC_RMD-draft"]        =  0x0073,  --draft-ietf-tls-openpgp-keys-05
+["TLS_DHE_DSS_WITH_AES_256_CBC_RMD-draft"]        =  0x0074,  --draft-ietf-tls-openpgp-keys-05
+["TLS_DHE_RSA_WITH_3DES_EDE_CBC_RMD-draft"]       =  0x0077,  --draft-ietf-tls-openpgp-keys-05
+["TLS_DHE_RSA_WITH_AES_128_CBC_RMD-draft"]        =  0x0078,  --draft-ietf-tls-openpgp-keys-05
+["TLS_DHE_RSA_WITH_AES_256_CBC_RMD-draft"]        =  0x0079,  --draft-ietf-tls-openpgp-keys-05
+["TLS_RSA_WITH_3DES_EDE_CBC_RMD-draft"]           =  0x007C,  --draft-ietf-tls-openpgp-keys-05
+["TLS_RSA_WITH_AES_128_CBC_RMD-draft"]            =  0x007D,  --draft-ietf-tls-openpgp-keys-05
+["TLS_RSA_WITH_AES_256_CBC_RMD-draft"]            =  0x007E,  --draft-ietf-tls-openpgp-keys-05
+["TLS_GOSTR341094_WITH_28147_CNT_IMIT-draft"]     =  0x0080,  --draft-chudov-cryptopro-cptls-04
+["TLS_GOSTR341001_WITH_28147_CNT_IMIT-draft"]     =  0x0081,  --draft-chudov-cryptopro-cptls-04
+["TLS_GOSTR341094_WITH_NULL_GOSTR3411-draft"]     =  0x0082,  --draft-chudov-cryptopro-cptls-04
+["TLS_GOSTR341001_WITH_NULL_GOSTR3411-draft"]     =  0x0083,  --draft-chudov-cryptopro-cptls-04
 ["TLS_RSA_WITH_CAMELLIA_256_CBC_SHA"]              =  0x0084,
 ["TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA"]           =  0x0085,
 ["TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA"]           =  0x0086,
@@ -398,7 +432,6 @@ CIPHERS = {
 ["TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA256"]       =  0x00BD,
 ["TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA256"]       =  0x00BE,
 ["TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA256"]       =  0x00BF,
-["TLS_EMPTY_RENEGOTIATION_INFO_SCSV"]              =  0x00FF,
 ["TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256"]           =  0x00C0,
 ["TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA256"]        =  0x00C1,
 ["TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA256"]        =  0x00C2,
@@ -583,6 +616,14 @@ CIPHERS = {
 ["SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA"]             =  0xFEFF,
 }
 
+DEFAULT_CIPHERS = {
+  "TLS_RSA_WITH_AES_128_CBC_SHA", -- mandatory TLSv1.2
+  "TLS_RSA_WITH_3DES_EDE_CBC_SHA", -- mandatory TLSv1.1
+  "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA", -- mandatory TLSv1.0
+  "TLS_DHE_RSA_WITH_AES_256_CBC_SHA", -- DHE with strong AES
+  "TLS_RSA_WITH_RC4_128_MD5", -- Weak and old, but likely supported on old stuff
+}
+
 local function find_key(t, value)
   local k, v
 
@@ -595,14 +636,580 @@ local function find_key(t, value)
   return nil
 end
 
+-- Keep this local to enforce use of the cipher_info function
+local cipher_info_cache = {
+  -- pre-populate the special cases that break the parser below
+  ["TLS_ECDH_anon_NULL_WITH_SHA-draft"] = {
+    kex = "ECDH", dh = true, ec = true,
+    server_auth = "anon",
+    cipher = "NULL",
+    hash = "SHA",
+    draft = true
+  },
+  ["TLS_ECMQV_ECDSA_NULL_SHA-draft"] = {
+    kex = "ECMQV", ec = true,
+    server_auth = "ECDSA",
+    cipher = "NULL",
+    hash = "SHA",
+    draft = true
+  },
+  ["TLS_ECMQV_ECNRA_NULL_SHA-draft"] = {
+    kex = "ECMQV", ec = true,
+    server_auth = "ECNRA",
+    cipher = "NULL",
+    hash = "SHA",
+    draft = true
+  },
+  ["TLS_GOSTR341094_WITH_28147_CNT_IMIT-draft"] = {
+    kex = "GOSTR341094",
+    server_auth = "GOSTR341094",
+    cipher = "GOST28147",
+    hash = "IMIT_GOST28147",
+    draft = true
+  },
+  ["TLS_GOSTR341001_WITH_28147_CNT_IMIT-draft"] = {
+    kex = "GOSTR341001",
+    server_auth = "GOSTR341001",
+    cipher = "GOST28147",
+    hash = "IMIT_GOST28147",
+    draft = true
+  },
+  ["TLS_GOSTR341094_WITH_NULL_GOSTR3411-draft"] = {
+    kex = "GOSTR341094",
+    server_auth = "GOSTR341094",
+    cipher = "NULL",
+    hash = "HMAC_GOSTR3411",
+    draft = true
+  },
+  ["TLS_GOSTR341001_WITH_NULL_GOSTR3411-draft"] = {
+    kex = "GOSTR341001",
+    server_auth = "GOSTR341001",
+    cipher = "NULL",
+    hash = "HMAC_GOSTR3411",
+    draft = true
+  },
+}
+
+
+-- A couple helpers for server_key_exchange parsing
+local function unpack_dhparams (blob, pos)
+  local p, g, y
+  p, g, y, pos = unpack(">s2s2s2", blob, pos)
+  return pos, {p=p, g=g, y=y}, #p * 8
+end
+
+local function unpack_ecdhparams (blob, pos)
+  local eccurvetype
+  eccurvetype, pos = unpack("B", blob, pos)
+  local ret = {}
+  local strength
+  if eccurvetype == 1 then
+    local p, a, b, base, order, cofactor
+    p, a, b, base, order, cofactor, pos = unpack("s1s1s1s1s1s1", blob, pos)
+    strength = math.log(order, 2)
+    ret.curve_params = {
+      ec_curve_type = "explicit_prime",
+      prime_p=p, curve={a=a, b=b}, base=base, order=order, cofactor=cofactor
+    }
+  elseif eccurvetype == 2 then
+    local p = {}
+    local m, basis
+    m, basis, pos = unpack(">I2B", blob, pos)
+    if basis == 1 then -- ec_trinomial
+      p.k, pos = unpack("s1", blob, pos)
+    elseif basis == 2 then -- ec_pentanomial
+      p.k1, p.k2, p.k3, pos = unpack("s1s1s1", blob, pos)
+    end
+    local a, b, base, order, cofactor
+    a, b, base, order, cofactor, pos = unpack("s1s1s1s1s1", blob, pos)
+    strength = math.log(order, 2)
+    ret.curve_params = {
+      ec_curve_type = "explicit_char2",
+      m=m, basis=basis, field=p, curve={a=a, b=b}, base=base, order=order, cofactor=cofactor
+    }
+  elseif eccurvetype == 3 then
+    local curve
+    curve, pos = unpack(">I2", blob, pos)
+    ret.curve_params = {
+      ec_curve_type = "namedcurve",
+      curve = find_key(ELLIPTIC_CURVES, curve)
+    }
+    local size = ret.curve_params.curve:match("(%d+)[rk]%d$")
+    if size then
+      strength = tonumber(size)
+    end
+  end
+  ret.public, pos = unpack("s1", blob, pos)
+  return pos, ret, strength
+end
+
+local function unpack_signed (blob, pos, protocol)
+  if pos > #blob then -- not-signed
+    return pos, nil
+  end
+  local hash_alg, sig_alg, sig
+  -- TLSv1.2 changed to allow arbitrary hash and sig algorithms
+  if protocol and PROTOCOLS[protocol] >= 0x0303 then
+    hash_alg, sig_alg, sig, pos = unpack(">BBs2", blob, pos)
+  else
+    sig, pos = unpack(">s2", blob, pos)
+  end
+  return pos, {hash_algorithm=hash_alg, signature_algorithm=sig_alg, signature=sig}
+end
+
+--- Get the strength-equivalent RSA key size
+--
+-- Based on NIST SP800-57 part 1 rev 3
+-- @param ktype Key type ("dh", "ec", "rsa", "dsa")
+-- @param bits Size of key in bits
+-- @return Size in bits of RSA key with equivalent strength
+function rsa_equiv (ktype, bits)
+  if ktype == "rsa" or ktype == "dsa" or ktype == "dh" then
+    return bits
+  elseif ktype == "ec" then
+    if bits < 160 then
+      return 512 -- Possibly down to 0, but details not published
+    elseif bits < 224 then
+      return 1024
+    elseif bits < 256 then
+      return 2048
+    elseif bits < 384 then
+      return 3072
+    elseif bits < 512 then
+      return 7680
+    else -- 512+
+      return 15360
+    end
+  end
+  return nil
+end
+
+KEX_ALGORITHMS = {}
+
+-- RFC 5246
+KEX_ALGORITHMS.NULL = { anon = true }
+KEX_ALGORITHMS.DH_anon = {
+  anon = true,
+  type = "dh",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    pos, ret.dhparams, ret.strength = unpack_dhparams(blob)
+    return ret
+  end
+}
+KEX_ALGORITHMS.DH_anon_EXPORT = {
+  anon=true,
+  export=true,
+  type = "dh",
+  server_key_exchange = KEX_ALGORITHMS.DH_anon.server_key_exchange
+}
+KEX_ALGORITHMS.ECDH_anon = {
+  anon=true,
+  type = "ec",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    pos, ret.ecdhparams, ret.strength = unpack_ecdhparams(blob)
+    return ret
+  end
+}
+KEX_ALGORITHMS.ECDH_anon_EXPORT = {
+  anon=true,
+  export=true,
+  type = "ec",
+  server_key_exchange = KEX_ALGORITHMS.ECDH_anon.server_key_exchange
+}
+
+KEX_ALGORITHMS.RSA = {
+  pubkey="rsa",
+}
+-- http://www-archive.mozilla.org/projects/security/pki/nss/ssl/fips-ssl-ciphersuites.html
+KEX_ALGORITHMS.RSA_FIPS = KEX_ALGORITHMS.RSA
+KEX_ALGORITHMS.RSA_EXPORT = {
+  export=true,
+  pubkey="rsa",
+  type = "rsa",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {rsa={}}
+    ret.rsa.modulus, ret.rsa.exponent, pos = unpack(">s2s2", blob)
+    pos, ret.signed = unpack_signed(blob, pos, protocol)
+    ret.strength = #ret.rsa.modulus
+    return ret
+  end
+}
+KEX_ALGORITHMS.RSA_EXPORT1024 = KEX_ALGORITHMS.RSA_EXPORT
+KEX_ALGORITHMS.DHE_RSA={
+  pubkey="rsa",
+  type = "dh",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    pos, ret.dhparams, ret.strength = unpack_dhparams(blob)
+    pos, ret.signed = unpack_signed(blob, pos, protocol)
+    return ret
+  end
+}
+KEX_ALGORITHMS.DHE_RSA_EXPORT={
+  export=true,
+  pubkey="rsa",
+  type = "dh",
+  server_key_exchange = KEX_ALGORITHMS.DHE_RSA.server_key_exchange
+}
+KEX_ALGORITHMS.DHE_DSS={
+  pubkey="dsa",
+  type = "dh",
+  server_key_exchange = KEX_ALGORITHMS.DHE_RSA.server_key_exchange
+}
+KEX_ALGORITHMS.DHE_DSS_EXPORT={
+  export=true,
+  pubkey="dsa",
+  type = "dh",
+  server_key_exchange = KEX_ALGORITHMS.DHE_RSA.server_key_exchange
+}
+KEX_ALGORITHMS.DHE_DSS_EXPORT1024 = KEX_ALGORITHMS.DHE_DSS_EXPORT1024
+
+KEX_ALGORITHMS.DH_DSS={
+  pubkey="dh",
+}
+KEX_ALGORITHMS.DH_DSS_EXPORT={
+  export=true,
+  pubkey="dh",
+}
+KEX_ALGORITHMS.DH_RSA={
+  pubkey="dh",
+}
+KEX_ALGORITHMS.DH_RSA_EXPORT={
+  export=true,
+  pubkey="dh",
+}
+
+KEX_ALGORITHMS.ECDHE_RSA={
+  pubkey="rsa",
+  type = "ec",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    pos, ret.ecdhparams, ret.strength = unpack_ecdhparams(blob)
+    pos, ret.signed = unpack_signed(blob, pos, protocol)
+    return ret
+  end
+}
+KEX_ALGORITHMS.ECDHE_ECDSA={
+  pubkey="ec",
+  type = "ec",
+  server_key_exchange = KEX_ALGORITHMS.ECDHE_RSA.server_key_exchange
+}
+KEX_ALGORITHMS.ECDH_ECDSA={
+  pubkey="ec",
+}
+KEX_ALGORITHMS.ECDH_RSA={
+  pubkey="ec",
+}
+
+-- draft-ietf-tls-ecc-00
+KEX_ALGORITHMS.ECDH_ECNRA={
+  pubkey="ec",
+}
+KEX_ALGORITHMS.ECMQV_ECDSA={
+  pubkey="ec",
+  type = "ecmqv",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    ret.mqvparams, pos = unpack("s1", blob)
+    return ret
+  end
+}
+KEX_ALGORITHMS.ECMQV_ECNRA={
+  pubkey="ec",
+}
+
+-- rfc4279
+KEX_ALGORITHMS.PSK = {
+  type = "psk",
+  server_key_exchange = function (blob, protocol)
+    local hint, pos = unpack(">s2", blob)
+    return {psk_identity_hint=hint}
+  end
+}
+KEX_ALGORITHMS.RSA_PSK = {
+  pubkey="rsa",
+  type = "psk",
+  server_key_exchange = KEX_ALGORITHMS.PSK.server_key_exchange
+}
+KEX_ALGORITHMS.DHE_PSK = {
+  type = "dh",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    ret.psk_identity_hint, pos = unpack(">s2", blob)
+    pos, ret.dhparams, ret.strength = unpack_dhparams(blob, pos)
+    return ret
+  end
+}
+--nomenclature change
+KEX_ALGORITHMS.PSK_DHE = KEX_ALGORITHMS.DHE_PSK
+
+--rfc5489
+KEX_ALGORITHMS.ECDHE_PSK={
+  type = "ec",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {}
+    ret.psk_identity_hint, pos = unpack(">s2", blob)
+    pos, ret.ecdhparams, ret.strength = unpack_ecdhparams(blob, pos)
+    return ret
+  end
+}
+
+-- RFC 5054
+KEX_ALGORITHMS.SRP_SHA = {
+  type = "srp",
+  server_key_exchange = function (blob, protocol)
+    local pos
+    local ret = {srp={}}
+    ret.srp.N, ret.srp.g, ret.srp.s, ret.srp.B, pos = unpack(">s2s2s1s2", blob)
+    pos, ret.signed = unpack_signed(blob, pos, protocol)
+    ret.strength = #ret.srp.N
+    return ret
+  end
+}
+KEX_ALGORITHMS.SRP_SHA_DSS = {
+  pubkey="dsa",
+  type = "srp",
+  server_key_exchange = KEX_ALGORITHMS.SRP_SHA.server_key_exchange
+}
+KEX_ALGORITHMS.SRP_SHA_RSA = {
+  pubkey="rsa",
+  type = "srp",
+  server_key_exchange = KEX_ALGORITHMS.SRP_SHA.server_key_exchange
+}
+
+-- RFC 6101
+KEX_ALGORITHMS.FORTEZZA_KEA={}
+
+-- RFC 4491
+KEX_ALGORITHMS.GOSTR341001={}
+KEX_ALGORITHMS.GOSTR341094={}
+
+-- RFC 2712
+KEX_ALGORITHMS.KRB5={}
+KEX_ALGORITHMS.KRB5_EXPORT={
+  export=true,
+}
+
+
+--- Get info about a cipher suite
+--
+--  Returned table has "kex", "cipher", "mode", "size", and
+--  "hash" keys, as well as boolean flag "draft". The "draft"
+--  flag is only supported for some suites that have different enumeration
+--  values in draft versus final RFC.
+-- @param c The cipher suite name, e.g. TLS_RSA_WITH_AES_128_GCM_SHA256
+-- @return A table of info as described above.
+function cipher_info (c)
+  local info = cipher_info_cache[c]
+  if info then return info end
+  info = {}
+  local tokens = stdnse.strsplit("_", c)
+  local i = 1
+  if tokens[i] ~= "TLS" and tokens[i] ~= "SSL" then
+    stdnse.debug2("cipher_info: Not a TLS ciphersuite: %s", c)
+    return nil
+  end
+  -- kex, cipher, size, mode, hash
+  i = i + 1
+  while tokens[i] and tokens[i] ~= "WITH" do
+    i = i + 1
+  end
+  info.kex = table.concat(tokens, "_", 2, i-1)
+
+  if tokens[i] and tokens[i] ~= "WITH" then
+    stdnse.debug2("cipher_info: Can't parse (no WITH): %s", c)
+    return nil
+  end
+
+  -- cipher
+  i = i + 1
+  local t = tokens[i]
+  info.cipher = t
+  if t == "3DES" then
+    i = i + 1 -- 3DES_EDE
+  end
+
+  -- key size
+  if t == "3DES" then -- NIST SP 800-57
+    info.size = 112
+  elseif t == "CHACHA20" then
+    info.size = 256
+  elseif t == "IDEA" then
+    info.size = 128
+  elseif t == "SEED" then
+    info.size = 128
+  elseif t == "FORTEZZA" then
+    info.size = 80
+  elseif t == "DES" then
+    info.size = 56
+  elseif t == "RC2" or t == "DES40" then
+    info.size = 40
+  elseif t == "NULL" then
+    info.size = 0
+  else
+    i = i + 1
+    info.size = tonumber(tokens[i])
+  end
+
+  -- block size (bits)
+  if t == "3DES" or t == "RC2" or t == "IDEA" or t == "DES" or t == "FORTEZZA" or t == "DES40" then
+    info.block_size = 64
+  elseif t == "AES" or t == "CAMELLIA" or t == "ARIA" or t == "SEED" then
+    info.block_size = 128
+  end
+
+  -- stream ciphers don't have a mode
+  if info.cipher == "RC4" then
+    info.mode = "stream"
+  elseif info.cipher == "CHACHA20" then
+    i = i + 1
+    info.cipher = "CHACHA20-POLY1305"
+    info.mode = "stream"
+  elseif info.cipher ~= "NULL" then
+    i = i + 1
+    info.mode = tokens[i]
+  end
+
+  -- export key size override
+  if info.export and tonumber(tokens[i+1]) then
+    i = i + 1
+    info.size = tonumber(tokens[i])
+  end
+
+  -- Other key size overrides
+  if info.cipher == "RC4" then -- RFC 7465 prohibits RC4 in TLS
+    info.size = math.min(info.size or 80, 80) -- Equivalently caps to C grade?
+  end
+
+  -- hash
+  if info.mode == "CCM" then
+    info.hash = "SHA256"
+  else
+    i = i + 1
+    t = (tokens[i]):match("(.*)%-draft$")
+    if t then
+      info.draft = true
+    else
+      t = tokens[i]
+    end
+    info.hash = t
+  end
+
+  cipher_info_cache[c] = info
+  return info
+end
+
+SCSVS = {
+["TLS_EMPTY_RENEGOTIATION_INFO_SCSV"]              =  0x00FF, -- rfc5746
+["TLS_FALLBACK_SCSV"]                              =  0x5600, -- draft-ietf-tls-downgrade-scsv-00
+}
+
+message_parse = {
+    alert = function (buffer, j)
+      local b = {}
+      -- Parse body.
+      b.level, b.description, j = unpack("BB", buffer, j)
+
+      -- Convert to human-readable form.
+      b["level"] = find_key(TLS_ALERT_LEVELS, b["level"])
+      b["description"] = find_key(TLS_ALERT_REGISTRY, b["description"])
+
+      return b, j
+    end,
+    handshake = function (buffer, j, protocol)
+      -- Check for message fragmentation.
+      -- Need 4 bytes for message header with length
+      local have = #buffer - j + 1
+      if have < 4 then
+        return nil, j, 4
+      end
+
+      -- Parse body.
+      local b = {}
+      local len
+      b.type, len, j = unpack("B>I3", buffer, j)
+      local msg_end = len + j
+
+      -- Convert to human-readable form.
+      b["type"] = find_key(TLS_HANDSHAKETYPE_REGISTRY, b["type"])
+
+      -- Check for message fragmentation.
+      -- Need 4 bytes for header plus length of message
+      if have < len + 4 then
+        return nil, j - 4, len + 4
+      end
+
+      if b["type"] == "server_hello" then
+        -- Parse body.
+        b.protocol, b.time, b.random, b.session_id, j = unpack(">I2 I4 c28 s1", buffer, j)
+        b.cipher, b.compressor, j = unpack(">I2 B", buffer, j)
+        -- Optional extensions for TLS only
+        if j < msg_end and protocol ~= "SSLv3" then
+          local num_exts
+          b["extensions"] = {}
+          num_exts, j = unpack(">I2", buffer, j)
+          for e = 0, num_exts do
+            if j >= msg_end then break end
+            local extcode, datalen
+            extcode, j = unpack(">I2", buffer, j)
+            extcode = find_key(EXTENSIONS, extcode) or extcode
+            b["extensions"][extcode], j = unpack(">s2", buffer, j)
+          end
+        end
+
+        -- Convert to human-readable form.
+        b["protocol"] = find_key(PROTOCOLS, b["protocol"])
+        b["cipher"] = find_key(CIPHERS, b["cipher"])
+        b["compressor"] = find_key(COMPRESSORS, b["compressor"])
+      elseif b["type"] == "certificate" then
+        local cert_end
+        cert_end, j = unpack(">I3", buffer, j)
+        cert_end = cert_end + j
+        if cert_end > msg_end then
+          stdnse.debug2("server_certificate length > handshake body length!")
+        end
+        b["certificates"] = {}
+        while j < cert_end do
+          local cert_len, cert
+          cert, j = unpack(">s3", buffer, j)
+          -- parse these with sslcert.parse_ssl_certificate
+          table.insert(b["certificates"], cert)
+        end
+      else
+        -- TODO: implement other handshake message types
+        stdnse.debug2("Unknown handshake message type: %s", b["type"])
+        b.data, j = unpack("c" .. msg_end - j, buffer, j)
+      end
+
+      return b, j
+    end,
+    heartbeat = function (buffer, j)
+      local b = {}
+      b.type, b.payload, j = unpack(">B s2", buffer, j)
+      -- Heartbeat messages are one per record; consume the rest of the record as padding.
+      b.padding = buffer:sub(j)
+      return b, #buffer + 1
+    end,
+}
+
 ---
 -- Read a SSL/TLS record
--- @param buffer The read buffer
--- @param i      The position in the buffer to start reading
+-- @param buffer   The read buffer
+-- @param i        The position in the buffer to start reading
+-- @param fragment Message fragment left over from previous record (nil if none)
 -- @return The current position in the buffer
 -- @return The record that was read, as a table
-function record_read(buffer, i)
-  local b, h, len
+function record_read(buffer, i, fragment)
 
   ------------
   -- Header --
@@ -614,29 +1221,36 @@ function record_read(buffer, i)
   end
 
   -- Parse header.
-  h = {}
-  local j, typ, proto = bin.unpack(">CS", buffer, i)
+  local h = {}
+  local typ, proto, rlength, j = unpack(">B I2 I2", buffer, i)
+  h.length = rlength
   local name = find_key(TLS_CONTENTTYPE_REGISTRY, typ)
   if name == nil then
-    stdnse.print_debug("Unknown TLS ContentType: %d", typ)
+    stdnse.debug1("Unknown TLS ContentType: %d", typ)
     return j, nil
   end
   h["type"] = name
   name = find_key(PROTOCOLS, proto)
   if name == nil then
-    stdnse.print_debug("Unknown TLS Protocol: 0x%x", typ)
+    stdnse.debug1("Unknown TLS Protocol: 0x%04x", proto)
     return j, nil
   end
   h["protocol"] = name
 
-  j, h["length"] = bin.unpack(">S", buffer, j)
-
   -- Ensure we have enough data for the body.
-  len = j + h["length"] - 1
-  if #buffer < len then
+  if #buffer < j + rlength - 1 then
     return i, nil
   end
 
+  -- Adjust buffer and length to account for message fragment left over
+  -- from last record.
+  local mbuffer
+  if fragment then
+    mbuffer = fragment .. buffer:sub(j, j + rlength)
+  else
+    mbuffer = buffer:sub(j, j + rlength)
+  end
+
   -- Convert to human-readable form.
 
   ----------
@@ -644,73 +1258,32 @@ function record_read(buffer, i)
   ----------
 
   h["body"] = {}
-  while j < len do
+
+  local mi = 1
+  while mi < #mbuffer do
     -- RFC 2246, 6.2.1 "multiple client messages of the same ContentType may
     -- be coalesced into a single TLSPlaintext record"
-    -- TODO: implement reading of fragmented records
-    b = {}
-    table.insert(h["body"], b)
-    if h["type"] == "alert" then
-      -- Parse body.
-      j, b["level"] = bin.unpack("C", buffer, j)
-      j, b["description"] = bin.unpack("C", buffer, j)
-
-      -- Convert to human-readable form.
-      b["level"] = find_key(TLS_ALERT_LEVELS, b["level"])
-      b["description"] = find_key(TLS_ALERT_REGISTRY, b["description"])
-    elseif h["type"] == "handshake" then
-      -- Parse body.
-      j, b["type"] = bin.unpack("C", buffer, j)
-      local blen, blen_upper
-      j, blen_upper, blen = bin.unpack("C>S", buffer, j)
-      blen = blen + blen_upper * 0x10000
-      local msg_end = j + blen
-
-      -- Convert to human-readable form.
-      b["type"] = find_key(TLS_HANDSHAKETYPE_REGISTRY, b["type"])
-
-      if b["type"] == "server_hello" then
-        -- Parse body.
-        j, b["protocol"] = bin.unpack(">S", buffer, j)
-        j, b["time"] = bin.unpack(">I", buffer, j)
-        j, b["random"] = bin.unpack("A28", buffer, j)
-        j, b["session_id_length"] = bin.unpack("C", buffer, j)
-        j, b["session_id"] = bin.unpack("A" .. b["session_id_length"], buffer, j)
-        j, b["cipher"] = bin.unpack(">S", buffer, j)
-        j, b["compressor"] = bin.unpack("C", buffer, j)
-        -- Optional extensions for TLS only
-        if j < msg_end and h["protocol"] ~= "SSLv3" then
-          local num_exts
-          b["extensions"] = {}
-          j, num_exts = bin.unpack(">S", buffer, j)
-          for e = 0, num_exts do
-            if j >= msg_end then break end
-            local extcode, datalen
-            j, extcode = bin.unpack(">S", buffer, j)
-            extcode = find_key(EXTENSIONS, extcode) or extcode
-            j, b["extensions"][extcode] = bin.unpack(">P", buffer, j)
-          end
-        end
-
-        -- Convert to human-readable form.
-        b["protocol"] = find_key(PROTOCOLS, b["protocol"])
-        b["cipher"] = find_key(CIPHERS, b["cipher"])
-        b["compressor"] = find_key(COMPRESSORS, b["compressor"])
-      else
-        -- TODO: implement other handshake message types
-        stdnse.print_debug(2, "Unknown handshake message type: %s", b["type"])
-        j = msg_end
-      end
-    elseif h["type"] == "heartbeat" then
-      j, b["type"], b["payload_length"] = bin.unpack("C>S", buffer, j)
-      j, b["payload"], b["padding"] = bin.unpack("PP", buffer, j)
-    else
-      stdnse.print_debug("Unknown message type: %s", h["type"])
+    local parser = message_parse[h.type]
+    if not parser then
+      stdnse.debug1("Unknown message type: %s", h["type"])
+      break
+    end
+    local b, need
+    b, mi, need = parser(mbuffer, mi, h.protocol)
+    if b then
+      table.insert(h.body, b)
+    elseif need then
+      stdnse.debug1("Fragmented after %d messages in %d bytes. %d remaining, need %d",
+        #h.body, mi, #mbuffer - mi + 1, need)
+      -- Fragmented message
+      h.fragment = mbuffer:sub(mi)
+      break
     end
   end
 
-  -- Ignore unparsed bytes.
-  j = len+1
+  -- Skip to the end of the record. Ignore unparsed bytes.
+  -- These should be handled as fragmentation above
+  j = j + rlength
 
   return j, h
 end
@@ -724,15 +1297,32 @@ end
 function record_write(type, protocol, b)
   return table.concat({
     -- Set the header as a handshake.
-    bin.pack("C", TLS_CONTENTTYPE_REGISTRY[type]),
+    pack("B", TLS_CONTENTTYPE_REGISTRY[type]),
     -- Set the protocol.
-    bin.pack(">S", PROTOCOLS[protocol]),
+    pack(">I2", PROTOCOLS[protocol]),
     -- Set the length of the header body.
-    bin.pack(">S", #b),
-    b
+    pack(">s2", b)
   })
 end
 
+-- Claim to support every hash and signature algorithm combination (TLSv1.2 only)
+--
+local signature_algorithms_all
+do
+  local sigalgs = {}
+  for hash, _ in pairs(HashAlgorithms) do
+    for sig, _ in pairs(SignatureAlgorithms) do
+      -- RFC 5246 7.4.1.4.1.
+      -- The "anonymous" value is meaningless in this context but used in
+      -- Section 7.4.3.  It MUST NOT appear in this extension.
+      if sig ~= "anonymous" then
+        sigalgs[#sigalgs+1] = {hash, sig}
+      end
+    end
+  end
+  signature_algorithms_all = EXTENSION_HELPERS["signature_algorithms"](sigalgs)
+end
+
 ---
 -- Build a client_hello message
 --
@@ -745,6 +1335,7 @@ end
 -- @return The client_hello record as a string
 function client_hello(t)
   local b, ciphers, compressor, compressors, h, len
+  t = t or {}
 
   ----------
   -- Body --
@@ -752,29 +1343,33 @@ function client_hello(t)
 
   b = {}
   -- Set the protocol.
-  table.insert(b, bin.pack(">S", PROTOCOLS[t["protocol"]]))
-
-  -- Set the random data.
-  table.insert(b, bin.pack(">I", os.time()))
+  local protocol = t["protocol"] or HIGHEST_PROTOCOL
+  table.insert(b, pack(">I2 I4",
+    PROTOCOLS[protocol],
+    -- Set the random data.
+    os.time()
+  ))
 
   -- Set the random data.
   table.insert(b, stdnse.generate_random_string(28))
 
   -- Set the session ID.
-  table.insert(b, bin.pack("C", 0))
+  table.insert(b, '\0')
 
   -- Cipher suites.
   ciphers = {}
-  if t["ciphers"] ~= nil then
-    -- Add specified ciphers.
-    for _, cipher in pairs(t["ciphers"]) do
-      table.insert(ciphers, bin.pack(">S", CIPHERS[cipher]))
+  -- Add specified ciphers.
+  for _, cipher in pairs(t["ciphers"] or DEFAULT_CIPHERS) do
+    if type(cipher) == "string" then
+      cipher = CIPHERS[cipher] or SCSVS[cipher]
+    end
+    if type(cipher) == "number" and cipher >= 0 and cipher <= 0xffff then
+      table.insert(ciphers, pack(">I2", cipher))
+    else
+      stdnse.debug1("Unknown cipher in client_hello: %s", cipher)
     end
-  else
-    -- Use NULL cipher
-    table.insert(ciphers, bin.pack(">S", CIPHERS["TLS_NULL_WITH_NULL_NULL"]))
   end
-  table.insert(b, bin.pack(">P", table.concat(ciphers)))
+  table.insert(b, pack(">s2", table.concat(ciphers)))
 
   -- Compression methods.
   compressors = {}
@@ -782,32 +1377,40 @@ function client_hello(t)
     -- Add specified compressors.
     for _, compressor in pairs(t["compressors"]) do
       if compressor ~= "NULL" then
-        table.insert(compressors, bin.pack("C", COMPRESSORS[compressor]))
+        table.insert(compressors, pack("B", COMPRESSORS[compressor]))
       end
     end
   end
   -- Always include NULL as last choice
-  table.insert(compressors, bin.pack("C", COMPRESSORS["NULL"]))
-  table.insert(b, bin.pack(">p", table.concat(compressors)))
+  table.insert(compressors, pack("B", COMPRESSORS["NULL"]))
+  table.insert(b, pack("s1", table.concat(compressors)))
 
   -- TLS extensions
-  if PROTOCOLS[t["protocol"]] and
-      PROTOCOLS[t["protocol"]] ~= PROTOCOLS["SSLv3"] then
+  if PROTOCOLS[protocol] and protocol ~= "SSLv3" then
     local extensions = {}
     if t["extensions"] ~= nil then
+      -- Do we need to add the signature_algorithms extension?
+      local need_sigalg = (protocol == "TLSv1.2")
       -- Add specified extensions.
       for extension, data in pairs(t["extensions"]) do
         if type(extension) == "number" then
-          table.insert(extensions, bin.pack(">S", extension))
+          table.insert(extensions, pack(">I2", extension))
         else
-          table.insert(extensions, bin.pack(">S", EXTENSIONS[extension]))
+          if extension == "signature_algorithms" then
+            need_sigalg = false
+          end
+          table.insert(extensions, pack(">I2", EXTENSIONS[extension]))
         end
-        table.insert(extensions, bin.pack(">P", data))
+        table.insert(extensions, pack(">s2", data))
+      end
+      if need_sigalg then
+        table.insert(extensions, pack(">I2", EXTENSIONS["signature_algorithms"]))
+        table.insert(extensions, pack(">s2", signature_algorithms_all))
       end
     end
     -- Extensions are optional
     if #extensions ~= 0 then
-      table.insert(b, bin.pack(">P", table.concat(extensions)))
+      table.insert(b, pack(">s2", table.concat(extensions)))
     end
   end
 
@@ -820,16 +1423,13 @@ function client_hello(t)
   h = {}
 
   -- Set type to ClientHello.
-  table.insert(h, bin.pack("C", TLS_HANDSHAKETYPE_REGISTRY["client_hello"]))
+  table.insert(h, pack("B", TLS_HANDSHAKETYPE_REGISTRY["client_hello"]))
 
   -- Set the length of the body.
-  len = bin.pack(">I", #b)
-  -- body length is 24 bits big-endian, so the 3 LSB of len
-  table.insert(h, len:sub(2,4))
+  table.insert(h, pack(">s3", b))
 
-  table.insert(h, b)
-
-  return record_write("handshake", t["protocol"], table.concat(h))
+  -- Record layer version should be SSLv3 (lowest compatible record version)
+  return record_write("handshake", "SSLv3", table.concat(h))
 end
 
 local function read_atleast(s, n)
@@ -871,7 +1471,7 @@ function record_buffer(sock, buffer, i)
     count = count + #resp
   end
   -- ContentType, ProtocolVersion, length
-  local _, _, _, len = bin.unpack(">CSS", buffer, i)
+  local _, _, len = unpack(">BI2I2", buffer, i)
   if count < TLS_RECORD_HEADER_LENGTH + len then
     status, resp = read_atleast(sock, TLS_RECORD_HEADER_LENGTH + len - count)
     if not status then
