@@ -9,7 +9,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -125,7 +125,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc 34798 2015-06-30 04:04:51Z dmiller $ */
+/* $Id: output.cc 36298 2016-09-21 03:55:12Z dmiller $ */
 
 #include "nmap.h"
 #include "output.h"
@@ -462,7 +462,8 @@ static std::string escape_for_screen(const std::string s) {
   for (unsigned int i = 0; i < s.size(); i++) {
     char buf[5];
     unsigned char c = s[i];
-    if (c == '\t' || c == '\r' || c == '\n' || (0x20 <= c && c <= 0x7e)) {
+    // Printable and some whitespace ok. "\r" not ok because it overwrites the line.
+    if (c == '\t' || c == '\n' || (0x20 <= c && c <= 0x7e)) {
       r += c;
     } else {
       Snprintf(buf, sizeof(buf), "\\x%02X", c);
@@ -796,15 +797,16 @@ void printportoutput(Target *currenths, PortList *plist) {
           *p = '|';
           p++;
         }
-        if (!sd.name)
-          serviceinfo[0] = '\0';
-        else {
+        if (sd.name || sd.service_fp || sd.service_tunnel != SERVICE_TUNNEL_NONE) {
           p = serviceinfo;
           while ((p = strchr(p, '/'))) {
             *p = '|';
             p++;
           }
         }
+        else {
+          serviceinfo[0] = '\0';
+        }
         log_write(LOG_MACHINE, "%d/%s/%s//%s//%s/", current->portno,
                   state, protocol, serviceinfo, grepvers);
 
@@ -986,9 +988,6 @@ void log_vwrite(int logt, const char *fmt, va_list ap) {
       case LOG_MACHINE:
       case LOG_SKID:
       case LOG_XML:
-        len = alloc_vsprintf(&writebuf, fmt, ap);
-        if (writebuf == NULL)
-          fatal("%s: alloc_vsprintf failed.", __func__);
         if (logtype == LOG_SKID_NOXLT)
             l = LOG_SKID;
         else
@@ -999,17 +998,22 @@ void log_vwrite(int logt, const char *fmt, va_list ap) {
           l >>= 1;
         }
         assert(fileidx < LOG_NUM_FILES);
-        if (o.logfd[fileidx] && len) {
-          if ((logtype & (LOG_SKID|LOG_SKID_NOXLT)) && !skid_noxlate)
-            skid_output(writebuf);
+        if (o.logfd[fileidx]) {
+          len = alloc_vsprintf(&writebuf, fmt, ap);
+          if (writebuf == NULL)
+            fatal("%s: alloc_vsprintf failed.", __func__);
+          if (len) {
+            if ((logtype & (LOG_SKID|LOG_SKID_NOXLT)) && !skid_noxlate)
+              skid_output(writebuf);
 
-          rc = fwrite(writebuf, len, 1, o.logfd[fileidx]);
-          if (rc != 1) {
-            fatal("Failed to write %d bytes of data to (logt==%d) stream. fwrite returned %d.  Quitting.", len, logtype, rc);
+            rc = fwrite(writebuf, len, 1, o.logfd[fileidx]);
+            if (rc != 1) {
+              fatal("Failed to write %d bytes of data to (logt==%d) stream. fwrite returned %d.  Quitting.", len, logtype, rc);
+            }
+            va_end(apcopy);
           }
-          va_end(apcopy);
+          free(writebuf);
         }
-        free(writebuf);
         break;
 
       default:
@@ -1878,7 +1882,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "open");
     xml_attribute("proto", "tcp");
-    xml_attribute("portid", "%hu", FPR->osscan_opentcpport);
+    xml_attribute("portid", "%d", FPR->osscan_opentcpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1886,7 +1890,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "closed");
     xml_attribute("proto", "tcp");
-    xml_attribute("portid", "%hu", FPR->osscan_closedtcpport);
+    xml_attribute("portid", "%d", FPR->osscan_closedtcpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1894,7 +1898,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "closed");
     xml_attribute("proto", "udp");
-    xml_attribute("portid", "%hu", FPR->osscan_closedudpport);
+    xml_attribute("portid", "%d", FPR->osscan_closedudpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1988,17 +1992,17 @@ void printosscanoutput(Target *currenths) {
   if (currenths->seq.lastboot) {
     char tmbuf[128];
     struct timeval tv;
-    time_t lastboot;
-    lastboot = (time_t) currenths->seq.lastboot;
-    strncpy(tmbuf, ctime(&lastboot), sizeof(tmbuf));
+    double uptime;
+    strncpy(tmbuf, ctime(&currenths->seq.lastboot), sizeof(tmbuf));
     chomp(tmbuf);
     gettimeofday(&tv, NULL);
+    uptime = difftime(tv.tv_sec, currenths->seq.lastboot);
     if (o.verbose)
       log_write(LOG_PLAIN, "Uptime guess: %.3f days (since %s)\n",
-                (double) (tv.tv_sec - currenths->seq.lastboot) / 86400,
+                uptime / 86400,
                 tmbuf);
     xml_open_start_tag("uptime");
-    xml_attribute("seconds", "%li", tv.tv_sec - currenths->seq.lastboot);
+    xml_attribute("seconds", "%.0f", uptime);
     xml_attribute("lastboot", "%s", tmbuf);
     xml_close_empty_tag();
     xml_newline();
@@ -2094,7 +2098,7 @@ void printserviceinfooutput(Target *currenths) {
   Port port;
   struct serviceDeductions sd;
   int i, numhostnames = 0, numostypes = 0, numdevicetypes = 0, numcpes = 0;
-  char hostname_tbl[MAX_SERVICE_INFO_FIELDS][MAXHOSTNAMELEN];
+  char hostname_tbl[MAX_SERVICE_INFO_FIELDS][FQDN_LEN+1];
   char ostype_tbl[MAX_SERVICE_INFO_FIELDS][64];
   char devicetype_tbl[MAX_SERVICE_INFO_FIELDS][64];
   char cpe_tbl[MAX_SERVICE_INFO_FIELDS][80];
@@ -2483,7 +2487,7 @@ void print_xml_finished_open(time_t timep, const struct timeval *tv) {
   xml_attribute("timestr", "%s", mytime);
   xml_attribute("elapsed", "%.2f", o.TimeSinceStart(tv));
   xml_attribute("summary",
-    "Nmap done at %s; %d %s (%d %s up) scanned in %.2f seconds",
+    "Nmap done at %s; %u %s (%u %s up) scanned in %.2f seconds",
     mytime, o.numhosts_scanned,
     (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
     o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
@@ -2528,7 +2532,7 @@ void printfinaloutput() {
   }
 
   log_write(LOG_STDOUT | LOG_SKID,
-            "Nmap done: %d %s (%d %s up) scanned in %.2f seconds\n",
+            "Nmap done: %u %s (%u %s up) scanned in %.2f seconds\n",
             o.numhosts_scanned,
             (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
             o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
@@ -2550,7 +2554,7 @@ void printfinaloutput() {
   xml_newline();
 
   log_write(LOG_NORMAL | LOG_MACHINE,
-            "# Nmap done at %s -- %d %s (%d %s up) scanned in %.2f seconds\n",
+            "# Nmap done at %s -- %u %s (%u %s up) scanned in %.2f seconds\n",
             mytime, o.numhosts_scanned,
             (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
             o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
