@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -122,7 +122,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: scan_engine.h 34806 2015-06-30 18:34:28Z dmiller $ */
+/* $Id: scan_engine.h 36133 2016-08-19 12:38:29Z dmiller $ */
 
 #ifndef SCAN_ENGINE_H
 #define SCAN_ENGINE_H
@@ -135,6 +135,8 @@
 #include "tcpip.h"
 #include <list>
 #include <vector>
+#include <set>
+#include <algorithm>
 
 struct probespec_tcpdata {
   u16 dport;
@@ -649,6 +651,12 @@ struct ultra_scan_performance_vars : public scan_performance_vars {
   void init();
 };
 
+struct HssPredicate {
+public:
+  const int operator() (const HostScanStats *lhs, const HostScanStats *rhs) const;
+  static struct sockaddr_storage *ss;
+};
+
 class UltraScanInfo {
 public:
   UltraScanInfo();
@@ -730,11 +738,11 @@ public:
 
   /* Any function which messes with (removes elements from)
      incompleteHosts may have to manipulate nextI */
-  std::list<HostScanStats *> incompleteHosts;
+  std::set<HostScanStats *, HssPredicate> incompleteHosts;
   /* Hosts are moved from incompleteHosts to completedHosts as they are
      completed. We keep them around because sometimes responses come back very
      late, after we consider a host completed. */
-  std::list<HostScanStats *> completedHosts;
+  std::set<HostScanStats *, HssPredicate> completedHosts;
   /* How long (in msecs) we keep a host in completedHosts */
   unsigned int completedHostLifetime;
   /* The last time we went through completedHosts to remove hosts */
@@ -751,7 +759,7 @@ public:
 private:
 
   unsigned int numInitialTargets;
-  std::list<HostScanStats *>::iterator nextI;
+  std::set<HostScanStats *>::iterator nextI;
 
 };
 
